package com.flyer.foster.service.impl;

import cn.dev33.satoken.stp.StpUtil;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.flyer.exception.BusinessException;
import com.flyer.foster.dto.*;
import com.flyer.foster.entity.AppInfo;
import com.flyer.foster.entity.Farm;
import com.flyer.foster.entity.WorldMap;
import com.flyer.foster.mapper.IGardenMapper;
import com.flyer.foster.service.IAppInfoService;
import com.flyer.foster.service.IGardenService;
import com.flyer.foster.service.IWorldMapService;
import com.flyer.foster.util.HttpUtils;
import com.flyer.foster.util.WeChatApiUtil;
import com.flyer.foster.util.WxAccessToken;
import com.flyer.util.GeoCastUtil;
import okhttp3.Response;
import org.locationtech.jts.geom.Coordinate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 果园 服务实现类
 * </p>
 *
 * @author flyer
 * @since 2024-05-09
 */
@Service
public class GardenServiceImpl extends ServiceImpl<IGardenMapper, Farm> implements IGardenService {

    @Autowired
    private WeChatApiUtil weChatApiUtil;

    @Autowired
    private QiniuUploadService qiniuUploadService;

    @Autowired
    private IAppInfoService iAppInfoService;
    @Autowired
    private IWorldMapService iWorldMapService;

    @Override
    public IPage<GardenRespDTO> getPageList(IPage<GardenRespDTO> page, GardenQueryDTO queryDTO) {
        IPage<GardenRespDTO> pageList = baseMapper.getPageList(page, queryDTO);
        List<WorldMap> worldMaps = iWorldMapService.list();
        WorldMap worldMap = worldMaps.get(0);

        for (GardenRespDTO record : pageList.getRecords()) {
            Coordinate coordinate = GeoCastUtil.wktToGeom(record.getWkt()).getCoordinate();
            coordinate = GeoCastUtil.pixelToLatLng(coordinate.getX(), worldMap.getHeight() - coordinate.getY(), worldMap.getWidth(), worldMap.getHeight());
            record.setLatitude(coordinate.y);
            record.setLongitude(coordinate.x);
        }
        return pageList;
    }

    @Transactional
    @Override
    public boolean addGarden(GardenAddDTO addDTO) {
        int tenantId = StpUtil.getTokenSession().getInt("tenantId");
        Farm garden = new Farm();
        garden.setTenantId(tenantId);
        BeanUtil.copyProperties(addDTO, garden);
        this.save(garden);
        // 生成二维码
        String qrCode = this.generateQRCode(garden.getId(), addDTO.getAppid());

        return this.lambdaUpdate()
                .set(Farm::getQrCode, qrCode)
                .eq(Farm::getId, garden.getId()).update();
    }

    /**
     * 生成小程序二维码
     * @param gardenId 果园id
     */
    private String generateQRCode(Integer gardenId, String appId) {
        AppInfo appInfo = iAppInfoService.getByAppId(appId);
        WxAccessToken wxAccessToken = WxAccessToken.getInstance(weChatApiUtil, appInfo.getAppId(), appInfo.getAppSecret());
        StringBuilder sb2 = new StringBuilder(200);
        sb2.append("https://api.weixin.qq.com/cgi-bin/wxaapp/createwxaqrcode?access_token=");
        sb2.append(wxAccessToken.getAccess_token());
        Map<String, String> params = new HashMap<>();
        params.put("path", "/pages/subPages/login/index?gardenId=" + gardenId);
        params.put("width", "180");
        System.out.println(params.get("path"));
        Response response = HttpUtils.getResponse(sb2.toString(), params);
        if (response.isSuccessful()) {
            String key = "birdseye-look-mini/temp/" + System.currentTimeMillis() + "_" + gardenId + ".jpg";
            try {
                qiniuUploadService.upload(qiniuUploadService.uploadToken(), key, response.body().bytes());
                return key;
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return "";
    }

    @Transactional
    @Override
    public boolean deleteByIdList(List<Integer> idList) {
        return baseMapper.deleteByIdList(idList);
    }

    @Transactional
    @Override
    public boolean updateGarden(GardenUpdateDTO updateDTO) {
        Farm garden = baseMapper.selectById(updateDTO.getId());
        if (garden == null) {
            throw new BusinessException("无效的果园id");
        }
        // 生成二维码
        String qrCode = this.generateQRCode(garden.getId(), updateDTO.getAppid());
        return this.lambdaUpdate()
                .set(StrUtil.isNotBlank(updateDTO.getName()), Farm::getName, updateDTO.getName())
                .set(Farm::getQrCode, qrCode)
                .set(updateDTO.getWorldMapId() != null, Farm::getWorldMapId, updateDTO.getWorldMapId())
                .set(StrUtil.isNotBlank(updateDTO.getWkt()), Farm::getWkt, updateDTO.getWkt())
                .set(StrUtil.isNotBlank(updateDTO.getIcon()), Farm::getIcon, updateDTO.getIcon())
                .set(StrUtil.isNotBlank(updateDTO.getAppid()), Farm::getAppid, updateDTO.getAppid())
                .set(StrUtil.isNotBlank(updateDTO.getDayBg()), Farm::getDayBg, updateDTO.getDayBg())
                .set(StrUtil.isNotBlank(updateDTO.getNightBg()), Farm::getNightBg, updateDTO.getNightBg())
                .set(updateDTO.getBirdseyeGardenId() != null, Farm::getBirdseyeFarmId, updateDTO.getBirdseyeGardenId())
                .set(updateDTO.getUserId() != null, Farm::getUserId, updateDTO.getUserId())
                .set(StrUtil.isNotBlank(updateDTO.getBaseMapUrl()), Farm::getBaseMapUrl, updateDTO.getBaseMapUrl())
                .set(StrUtil.isNotBlank(updateDTO.getBgMusic()), Farm::getBgMusic, updateDTO.getBgMusic())
                .set(StrUtil.isNotBlank(updateDTO.getCoverUrl1()), Farm::getCoverUrl1, updateDTO.getCoverUrl1())
                .set(StrUtil.isNotBlank(updateDTO.getCoverUrl1Content()), Farm::getCoverUrl1Content, updateDTO.getCoverUrl1Content())
                .set(StrUtil.isNotBlank(updateDTO.getCoverUrl2()), Farm::getCoverUrl2, updateDTO.getCoverUrl2())
                .set(StrUtil.isNotBlank(updateDTO.getCoverUrl2Content()), Farm::getCoverUrl2Content, updateDTO.getCoverUrl2Content())
                .set(StrUtil.isNotBlank(updateDTO.getCoverUrl3()), Farm::getCoverUrl3, updateDTO.getCoverUrl3())
                .set(StrUtil.isNotBlank(updateDTO.getCoverUrl3Content()), Farm::getCoverUrl3Content, updateDTO.getCoverUrl3Content())
                .set(StrUtil.isNotBlank(updateDTO.getCoverUrl4()), Farm::getCoverUrl4, updateDTO.getCoverUrl4())
                .set(StrUtil.isNotBlank(updateDTO.getCoverUrl4Content()), Farm::getCoverUrl4Content, updateDTO.getCoverUrl4Content())
                .eq(Farm::getId, updateDTO.getId())
                .update();
    }
}
